@@ -31,19 +31,18 @@ class _CacheBuilder(object):
         self.reset_handlers()
 
 
-
     def reset_handlers(self):
         self._type_handlers = list()
 
     def register_handler(self, type_, hash_function, file_extension, load_function, save_function,
                          get_origin_function, set_origin_function):
         self._type_handlers.append((type_,
-                                    dill.dumps(hash_function),
+                                    self._pickler(hash_function),
                                     file_extension,
-                                    dill.dumps(load_function),
-                                    dill.dumps(save_function),
-                                    dill.dumps(get_origin_function),
-                                    dill.dumps(set_origin_function),
+                                    self._pickler(load_function),
+                                    self._pickler(save_function),
+                                    self._pickler(get_origin_function),
+                                    self._pickler(set_origin_function),
                                     )
                                    )
 
@@ -55,7 +54,8 @@ class _CacheBuilder(object):
 
         cache, lock, counter, handlers = self._setup_cache_internals()
 
-        c = _CachedFunction(function, folder, self.keep_in_memory, lock, cache, counter, handlers)
+        c = _CachedFunction(function, folder, self.keep_in_memory, lock, cache, counter, handlers,
+                            self._pickler, self._unpickler)
         return c
 
 
@@ -63,6 +63,8 @@ class CacheBuilder(_CacheBuilder):
 
     def __init__(self, root_dir=None, keep_in_memory=True):
         super(CacheBuilder, self).__init__(root_dir, keep_in_memory)
+        self._pickler = dill.dumps
+        self._unpickler = dill.loads
         self._manager = multiprocessing.Manager()
 
     def _setup_cache_internals(self):
@@ -96,6 +98,7 @@ class LocalCacheBuilder(_CacheBuilder):
     def __init__(self, root_dir=None, keep_in_memory=True):
         super(LocalCacheBuilder, self).__init__(root_dir, keep_in_memory)
         self._manager = None
+        self._pickler = self._unpickler = lambda o: o
 
 
     def _setup_cache_internals(self):
@@ -116,7 +119,8 @@ class _CachedFunction(object):
     runs.
     """
 
-    def __init__(self, function, folder, _keep_in_memory, _lock, _cache, _counter, _handlers):
+    def __init__(self, function, folder, _keep_in_memory, _lock, _cache, _counter, _handlers,
+            _pickler, _unpickler):
 
         self.function = function
         self.__name__ = function.__name__
@@ -127,6 +131,10 @@ class _CachedFunction(object):
         self._lock = _lock
         self._hit_counter = _counter
         self._type_handlers = _handlers
+
+        self._pickler = _pickler
+        self._unpickler = _unpickler
+
         self._logger = get_logger(self)
         self._setup_cache()
 
@@ -136,12 +144,12 @@ class _CachedFunction(object):
     def register_handler(self, type_, hash_function, file_extension, load_function, save_function,
                          get_origin_function, set_origin_function):
         self._type_handlers.append((type_,
-                                    dill.dumps(hash_function),
+                                    self._pickler(hash_function),
                                     file_extension,
-                                    dill.dumps(load_function),
-                                    dill.dumps(save_function),
-                                    dill.dumps(get_origin_function),
-                                    dill.dumps(set_origin_function),
+                                    self._pickler(load_function),
+                                    self._pickler(save_function),
+                                    self._pickler(get_origin_function),
+                                    self._pickler(set_origin_function),
                                     )
                                    )
 
@@ -149,27 +157,27 @@ class _CachedFunction(object):
         for row in self._type_handlers:
             if isinstance(obj, row[0]):
                 return row
-        nonf = dill.dumps(None)
+        nonf = self._pickler(None)
         return [None, nonf, None, nonf, nonf, nonf, nonf]
 
     def _lookup_hash_function_for(self, key):
-        return dill.loads(self._lookup_for_type_of(key)[1])
+        return self._unpickler(self._lookup_for_type_of(key)[1])
 
     def _lookup_load_function_for(self, ext):
         for (__, __, ext_i, load_function, __, __, __) in self._type_handlers:
             if ext == ext_i:
-                return dill.loads(load_function)
+                return self._unpickler(load_function)
         return None
 
     def _lookup_ext_and_save_function_for(self, what):
         row = self._lookup_for_type_of(what)
-        return row[2], dill.loads(row[4])
+        return row[2], self._unpickler(row[4])
 
     def _lookup_get_origin_function(self, what):
-        return dill.loads(self._lookup_for_type_of(what)[5])
+        return self._unpickler(self._lookup_for_type_of(what)[5])
 
     def _lookup_set_origin_function(self, what):
-        return dill.loads(self._lookup_for_type_of(what)[6])
+        return self._unpickler(self._lookup_for_type_of(what)[6])
 
     def get_number_of_hits(self):
         return self._hit_counter.value
@@ -199,9 +207,9 @@ class _CachedFunction(object):
         elif isinstance(key, basestring):
             data = key
         elif hasattr(key, "__dict__"):
-            data = dill.dumps(key.__dict__)
+            data = self._pickler(key.__dict__)
         elif isinstance(key, (bool, int, long, float,)):
-            data = dill.dumps(key)
+            data = self._pickler(key)
         else:
             raise Exception("can not compute hash for %r" % (key,))
         muncher = hashlib.sha1()
